@@ -1,18 +1,26 @@
+use 5.008;
 use strict;
 use warnings;
+use utf8;
 
 package File::ShareDir::ProjectDistDir;
-BEGIN {
-  $File::ShareDir::ProjectDistDir::AUTHORITY = 'cpan:KENTNL';
-}
-{
-  $File::ShareDir::ProjectDistDir::VERSION = '0.5.2';
-}
+
+our $VERSION = '1.000004';
 
 # ABSTRACT: Simple set-and-forget using of a '/share' directory in your projects root
 
+our $AUTHORITY = 'cpan:KENTNL'; # AUTHORITY
+
+
+
+
+
+
+
+
+
+
 
-use Path::Class::File;
 use Path::IsDev qw();
 use Path::FindDev qw(find_dev);
 use Sub::Exporter qw(build_exporter);
@@ -23,43 +31,185 @@ my ($exporter) = build_exporter(
     exports => [ dist_dir => \&build_dist_dir, dist_file => \&build_dist_file ],
     groups  => {
       all       => [qw( dist_dir dist_file )],
-      'default' => [qw( dist_dir dist_file )]
+      'default' => [qw( dist_dir dist_file )],
     },
     collectors => [ 'defaults', ],
-  }
+  },
 );
 my $env_key = 'FILE_SHAREDIR_PROJECTDISTDIR_DEBUG';
 
+## no critic (Subroutines::ProhibitSubroutinePrototypes)
+sub _debug($) { }
+## use critic
+
 if ( $ENV{$env_key} ) {
-  ## no critic (ProtectPrivateVars)
+  ## no critic (ProtectPrivateVars,TestingAndDebugging::ProhibitNoWarnings)
+  no warnings 'redefine';
   *File::ShareDir::ProjectDistDir::_debug = sub ($) {
     *STDERR->printf( qq{[ProjectDistDir] %s\n}, $_[0] );
   };
   $Path::IsDev::DEBUG   = 1;
   $Path::FindDev::DEBUG = 1;
 }
-else {
-  ## no critic (ProtectPrivateVars)
-  *File::ShareDir::ProjectDistDir::_debug = sub ($) { }
-}
 
 ## no critic (RequireArgUnpacking)
-sub _croak         { require Carp;              goto &Carp::croak }
-sub _path          { require Path::Tiny;        goto &Path::Tiny::path }
-sub _pathclassfile { require Path::Class::File; return Path::Class::File->new(@_) }
-sub _pathclassdir  { require Path::Class::Dir;  return Path::Class::Dir->new(@_) }
+sub _croak { require Carp; goto &Carp::croak }
+sub _carp  { require Carp; goto &Carp::carp }
+
+sub _path { require Path::Tiny; goto &Path::Tiny::path }
+
+sub _need_pathclass {
+  for my $package ( q[], q[::File], q[::Dir] ) {
+    ## no critic (Variables::RequireInitializationForLocalVars)
+    local $@ = undef;
+    my $code = sprintf 'require %s%s;1', 'Path::Class', $package;
+    ## no critic (BuiltinFunctions::ProhibitStringyEval,Lax::ProhibitStringyEval::ExceptForRequire)
+    next if eval $code;
+    my $err = $@;
+    _carp('Path::Class is not installed.');
+    ## no critic (RequireCarping, ErrorHandling::RequireUseOfExceptions)
+    die $err;
+  }
+  return 1;
+}
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 
 sub import {
   my ( $class, @args ) = @_;
-  my $has_defaults = undef;
 
-  my ( $xclass, $xfilename, $xline ) = caller;
+  my ( undef, $xfilename, undef ) = caller;
 
   my $defaults = {
     filename   => $xfilename,
     projectdir => 'share',
     pathclass  => undef,
+    strict     => undef,
   };
 
   if ( not @args ) {
@@ -71,13 +221,13 @@ sub import {
     my ( $key, $value );
     next unless $key = $args[$_] and $value = $args[ $_ + 1 ];
 
-    if ( $key eq 'defaults' ) {
+    if ( 'defaults' eq $key ) {
       $defaults = $value;
       undef $args[$_];
       undef $args[ $_ + 1 ];
       next;
     }
-    for my $setting (qw( projectdir filename distname pathclass pathtiny )) {
+    for my $setting (qw( projectdir filename distname pathclass pathtiny strict )) {
       if ( $key eq $setting and not ref $value ) {
         $defaults->{$setting} = $value;
         undef $args[$_];
@@ -90,29 +240,61 @@ sub import {
   $defaults->{filename}   = $xfilename if not defined $defaults->{filename};
   $defaults->{projectdir} = 'share'    if not defined $defaults->{projectdir};
 
+  if ( defined $defaults->{pathclass} ) {
+    _carp( 'Path::Class support depecated and will be removed from a future release.' . ' see Documentation for details' );
+    _need_pathclass();
+  }
+
   @_ = ( $class, ( grep { defined } @args ), 'defaults' => $defaults );
 
   goto $exporter;
 }
 
-sub _devel_sharedir {
-  my ( $filename, $subdir ) = @_;
 
-  _debug( 'Working on: ' . $filename );
-  my $dev = find_dev( _path($filename)->parent );
 
-  return if not defined $dev;
 
-  my $devel_share_dir = $dev->child($subdir);
-  if ( -d $devel_share_dir ) {
-    _debug( 'ISDEV : exists : <devroot>/' . $subdir . ' > ' . $devel_share_dir );
-    return $devel_share_dir;
-  }
-  _debug( 'ISPROD: does not exist : <devroot>/' . $subdir . ' > ' . $devel_share_dir );
 
-  #warn "Not a devel $dir";
-  return;
-}
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 
 sub _get_defaults {
@@ -129,67 +311,139 @@ sub _wrap_return {
     return $value unless ref $value;
     return "$value";
   }
-  if ( $type eq 'pathtiny' ) {
-    return $value if ref $value eq 'Path::Tiny';
-    return Path::Tiny::path($value);
+  if ( 'pathtiny' eq $type ) {
+    return $value if 'Path::Tiny' eq ref $value;
+    return _path($value);
   }
-  if ( $type eq 'pathclassdir' ) {
-    return $value if ref $value eq 'Path::Class::Dir';
-    require Path::Class::Dir;
+  if ( 'pathclassdir' eq $type ) {
+    return $value if 'Path::Class::Dir' eq ref $value;
+    _need_pathclass;
     return Path::Class::Dir->new("$value");
   }
-  if ( $type eq 'pathclassfile' ) {
-    return $value if ref $value eq 'Path::Class::File';
-    require Path::Class::File;
+  if ( 'pathclassfile' eq $type ) {
+    return $value if 'Path::Class::File' eq ref $value;
+    _need_pathclass;
     return Path::Class::File->new("$value");
   }
   return _croak("Unknown return type $type");
 }
 
+our %DIST_DIR_CACHE;
+
+sub _get_cached_dist_dir_result {
+  my ( undef, $filename, $projectdir, $distname, $strict ) = @_;
+  if ( defined $DIST_DIR_CACHE{$distname} ) {
+    return $DIST_DIR_CACHE{$distname};
+  }
+  _debug( 'Working on: ' . $filename );
+  my $dev = find_dev( _path($filename)->parent );
+
+  if ( not defined $dev ) {
+    ## no critic (Variables::ProhibitPackageVars)
+    local $Carp::CarpLevel = $Carp::CarpLevel + 1;
+    return File::ShareDir::dist_dir($distname);
+  }
+
+  my $devel_share_dir = $dev->child($projectdir);
+
+  if ($strict) {
+    $devel_share_dir = $devel_share_dir->child( 'dist', $distname );
+  }
+  if ( -d $devel_share_dir ) {
+    _debug( 'ISDEV : exists : <devroot>/' . $devel_share_dir->relative($dev) );
+    return ( $DIST_DIR_CACHE{$distname} = $devel_share_dir );
+  }
+  _debug( 'ISPROD: does not exist : <devroot>/' . $devel_share_dir->relative($dev) );
+  ## no critic (Variables::ProhibitPackageVars)
+  local $Carp::CarpLevel = $Carp::CarpLevel + 1;
+  return File::ShareDir::dist_dir($distname);
+}
+
 sub build_dist_dir {
-  my ( $class, $name, $arg, $col ) = @_;
+  my ( $class, undef, $arg, $col ) = @_;
 
   my $projectdir = _get_defaults( projectdir => $arg, $col );
   my $pathclass  = _get_defaults( pathclass  => $arg, $col );
   my $pathtiny   = _get_defaults( pathtiny   => $arg, $col );
+  my $strict     = _get_defaults( strict     => $arg, $col );
+  my $filename   = _get_defaults( filename   => $arg, $col );
 
   my $wrap_return_type;
 
   if ($pathclass) { $wrap_return_type = 'pathclassdir' }
   if ($pathtiny)  { $wrap_return_type = 'pathtiny' }
 
-  my $root = _devel_sharedir( $col->{defaults}->{filename}, $projectdir );
-
   my $distname = _get_defaults( distname => $arg, $col );
 
-  # In dev
-  if ($root) {
-    return sub { return _wrap_return( $wrap_return_type, $root ) };
-  }
-
-  # Non-Dev, no hardcoded distname
   if ( not $distname ) {
-    my $string_method = \&File::ShareDir::dist_dir;
-    return sub { return _wrap_return( $wrap_return_type, $string_method->(@_) ) };
+    return sub {
+      my ($udistname) = @_;
+      my $distdir = $class->_get_cached_dist_dir_result( $filename, $projectdir, $udistname, $strict );
+      return _wrap_return( $wrap_return_type, $distdir );
+    };
   }
-
-  # Non-Dev, hardcoded distname
-  my $string_method = sub() {
-    @_ = ($distname);
-    goto &File::ShareDir::dist_dir;
+  return sub {
+    my $distdir = $class->_get_cached_dist_dir_result( $filename, $projectdir, $distname, $strict );
+    return _wrap_return( $wrap_return_type, $distdir );
   };
-  return sub { return _wrap_return( $wrap_return_type, $string_method->(@_) ) };
 }
 
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 sub build_dist_file {
-  my ( $class, $name, $arg, $col ) = @_;
+  my ( $class, undef, $arg, $col ) = @_;
 
   my $projectdir = _get_defaults( projectdir => $arg, $col );
   my $pathclass  = _get_defaults( pathclass  => $arg, $col );
   my $pathtiny   = _get_defaults( pathtiny   => $arg, $col );
 
-  my $root = _devel_sharedir( $col->{defaults}->{filename}, $projectdir );
+  my $strict   = _get_defaults( strict   => $arg, $col );
+  my $filename = _get_defaults( filename => $arg, $col );
 
   my $distname = _get_defaults( distname => $arg, $col );
 
@@ -198,42 +452,29 @@ sub build_dist_file {
   if ($pathclass) { $wrap_return_type = 'pathclassfile' }
   if ($pathtiny)  { $wrap_return_type = 'pathtiny' }
 
-  if ($root) {
-    my $pathclass_method = sub {
-      my $file = ( $distname ? $_[0] : $_[1] );
-
-      # if the caller is devel, then we return the project root,
-      # regardless of what package you asked for.
-      # Might be bad, but we haven't imagined the scenario where yet.
-      my $path_o = $root->child($file)->absolute;
-      my $path_s = $path_o->stringify;
-      ## no critic ( ProhibitExplicitReturnUndef )
-      return undef unless -e $path_s;
-      if ( not -f $path_s ) {
-        return _croak("Found dist_file '$path_s', but not a file");
-      }
-      if ( not -r $path_s ) {
-        return _croak("File '$path_s', no read permissions");
-      }
-      return $path_o;
-    };
-    return sub {
-      return _wrap_return( $wrap_return_type, $pathclass_method->(@_) );
-    };
-  }
-  if ( not $distname ) {
-    my $string_method = \&File::ShareDir::dist_file;
-    return sub { return _wrap_return( $wrap_return_type, $string_method->(@_) ) };
-  }
-  my $string_method = sub($) {
-    if ( @_ != 1 or not defined $_[0] ) {
-      return _croak('dist_file takes only one argument,a filename, due to distname being specified during import');
+  my $check_file = sub {
+    my ( $distdir, $wanted_file ) = @_;
+    my $child = _path($distdir)->child($wanted_file);
+    return unless -e $child;
+    if ( -d $child ) {
+      return _croak("Found dist_file '$child', but is a dir");
     }
-    unshift @_, $distname;
-    goto &File::ShareDir::dist_file;
+    if ( not -r $child ) {
+      return _croak("File '$child', no read permissions");
+    }
+    return _wrap_return( $wrap_return_type, $child );
   };
+  if ( not $distname ) {
+    return sub {
+      my ( $udistname, $wanted_file ) = @_;
+      my $distdir = $class->_get_cached_dist_dir_result( $filename, $projectdir, $udistname, $strict );
+      return $check_file->( $distdir, $wanted_file );
+    };
+  }
   return sub {
-    return _wrap_return( $wrap_return_type, $string_method->(@_) );
+    my ($wanted_file) = @_;
+    my $distdir = $class->_get_cached_dist_dir_result( $filename, $projectdir, $distname, $strict );
+    return $check_file->( $distdir, $wanted_file );
   };
 }
 
@@ -243,13 +484,15 @@ __END__
 
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 File::ShareDir::ProjectDistDir - Simple set-and-forget using of a '/share' directory in your projects root
 
 =head1 VERSION
 
-version 0.5.2
+version 1.000004
 
 =head1 SYNOPSIS
 
@@ -280,7 +523,8 @@ you install that, you specify the different directory there also ) as follows:
 
     use File::ShareDir::ProjectDistDir (@args);
 
-This uses L<< C<Sub::Exporter>|Sub::Exporter >> to do the heavy lifting, so most usage of this module can be maximised by understanding that first.
+This uses L<< C<Sub::Exporter>|Sub::Exporter >> to do the heavy lifting, so most usage of this module can be maximized by
+understanding that first.
 
 =over 4
 
@@ -296,7 +540,7 @@ Import both C<dist_dir> and C<dist_file>
 
 Import the dist_dir method
 
-=item * B<C<dist_dir>>
+=item * B<C<dist_file>>
 
     ->import('dist_file' , .... )
 
@@ -340,6 +584,26 @@ i.e:
     my $dir = dist_dir();
     my $file = dist_file('path/to/file.pm' );
 
+=item * B<C<strict>>
+
+    ->import( ... , strict => 1 );
+
+This parameter specifies that all C<dist> C<sharedirs> will occur within the C<projectdir> directory using the following layout:
+
+    <root>/<projectdir>/dist/<DISTNAME>/
+
+As opposed to
+
+    <root>/<projectdir>
+
+This means if Heuristics misfire and accidentally find another distributions C<share> directory, it will not pick up on it
+unless that C<share> directory also has that layout, and will instead revert to the C<installdir> path in C<@INC>
+
+B<This parameter may become the default option in the future>
+
+Specifying this parameter also mandates you B<MUST> declare the C<DISTNAME> value in your file somewhere. Doing otherwise is
+considered insanity anyway.
+
 =item * B<C<defaults>>
 
     ->import( ... , defaults => {
@@ -415,12 +679,13 @@ which is mostly used internally, and their corresponding other values are packed
 
     # And leverages Sub::Exporter to create 2 subs in your package.
 
-Generates the exported 'dist_dir' method. In development environments, the generated method will return
-a path to the development directories 'share' directory. In non-development environments, this simply returns
-C<File::ShareDir::dist_dir>.
+Generates the exported 'dist_dir' method. In development environments, the generated method will return a path to the
+development directories 'share' directory. In non-development environments, this simply returns C<File::ShareDir::dist_dir>.
 
-As a result of this, specifying the Distribution name is not required during development, however, it will
-start to matter once it is installed. This is a potential avenues for bugs if you happen to name it wrong.
+As a result of this, specifying the Distribution name is not required during development ( unless in C<strict> mode ), however,
+it will start to matter once it is installed. This is a potential avenues for bugs if you happen to name it wrong.
+
+In C<strict> mode, the distribution name is B<ALWAYS REQUIRED>, either at least at C<import> or C<dist_dir()> time.
 
 =head2 build_dist_file
 
@@ -477,6 +742,117 @@ Caveats as a result of package-name as stated in L</build_dist_dir> also apply t
 
 =head1 SIGNIFICANT CHANGES
 
+=head2 1.000000
+
+=head3 Strict Mode.
+
+=head4 Using Strict Mode
+
+    use File::ShareDir::ProjectDistDir ':all', strict => 1;
+    use File::ShareDir::ProjectDistDir 'dist_dir' => { defaults => { strict => 1 }};
+
+=head4 Why you should use strict mode
+
+Starting with C<1.000000>, there is a parameter C<strict> that changes
+how C<sharedir> resolution performs.
+
+Without strict:
+
+    lib/...
+    share/...
+
+With strict
+
+    lib/...
+    share/dist/Dist-Name-Here/...
+
+This technique greatly builds resilience to the long standing problem
+with "develop" vs "install" heuristic ambiguity.
+
+Here at least,
+
+    dist_dir('Dist-Name')
+
+Will instead fall back to
+
+    @INC/auto/share/dist/Dist-Name
+
+When
+
+    share/dist/Dist-Name
+
+Does not exist.
+
+This means if you have a layout like this:
+
+    <DEVROOT>/inc/<a local::lib path here>
+    <DEVROOT>/lib/<development files here>
+
+Then when C<Foo-Bar-Baz> is installed as:
+
+    <DEVROOT>/inc/lib/Foo/Bar/Baz.pm
+    <DEVROOT>/inc/lib/auto/share/dist/Foo-Bar-Baz
+
+Then C<Baz.pm> will not see the C<DEVROOT> and assume "Hey, this is development" and then proceed to try finding files in
+C<DEVROOT/share>
+
+Instead, C<DEVROOT> must have C<DEVROOT/share/dist/Foo-Bar-Baz> too, otherwise it reverts
+to C<DEVROOT/inc/lib/auto...>
+
+=head3 C<Path::Class> interfaces deprecated and dependency dropped.
+
+If you have any dependence on this function, now is the time to get yourself off it.
+
+=head4 Minimum Changes to stay with C<Path::Class> short term.
+
+As the dependency has been dropped on C<Path::Class>, if you have C<CPAN>
+modules relying on C<Path::Class> interface, you should now at a very minimum
+start declaring
+
+    { requires => "Path::Class" }
+
+This will keep your dist working, but will not be future proof against further changes.
+
+=head4 Staying with C<Path::Class> long term.
+
+Recommended approach if you want to stay using the C<Path::Class> interface:
+
+    use File::ShareDir::... etc
+    use Path::Class qw( dir file );
+
+    my $dir = dir( dist_dir('Dist-Name') );
+
+This should future-proof you against anything File::ShareDir may do in the future.
+
+=head3 C<Versioning Scheme arbitrary converted to float>
+
+This change is a superficial one, and should have no bearing on how significant you think this release is.
+
+It is a significant release, but the primary reason for the version change is simply to avoid compatibility issues in
+I<versions themselves>.
+
+However, outside that, C<x.y.z> semantics are still intended to be semi-meaningful, just with less C<.> and more C<0> ☺
+
+=head3 C<dev> path determination now deferred to call time instead of C<use>
+
+This was essentially a required change to make C<strict> mode plausible, because strict mode _requires_ the C<distname> to be
+known, even in the development environment.
+
+This should not have any user visible effects, but please, if you have any problems, file a bug.
+
+=head3 C<file> component determination wrested from C<File::ShareDir>.
+
+    dist_file('foo','bar')
+
+Is now simply sugar syntax for
+
+    path(dist_dir('foo'))->child('bar')
+
+This should have no side effects in your code, but please file any bugs you experience.
+
+( return value is still C<undef> if the file does not exist, and still C<croak>'s if the file is not a file, or unreadable, but
+these may both be subject to change )
+
 =head2 0.5.0 - Heuristics and Return type changes
 
 =head3 New C<devdir> heuristic
@@ -501,7 +877,8 @@ For instance, have a look in C</usr/>
 
 This would have the very bad side effect of anything installed in C</usr/lib> thinking its "in development".
 
-Fortunately, nobody seems to have hit this specific bug, which I suspect is due only to C</usr/lib> being a symbolic link on most x86_64 systems.
+Fortunately, nobody seems to have hit this specific bug, which I suspect is due only to C</usr/lib> being a symbolic link on most
+x86_64 systems.
 
 =item * C<lib> is also reasonably common within C<CPAN> package names.
 
@@ -547,9 +924,10 @@ Now you can also get C<Path::Tiny> objects back, by passing:
         qw( dist_dir dist_file ),
         defaults => { pathtiny => 1 };
 
-For the time being, you can still get Path::Class objects back, but its likely to be deprecated in future.
+B<< For the time being, you can still get Path::Class objects back, it is deprecated since 1.000000 >>
 
-( In fact, I may even make 2 specific sub-classes of C<PDD> for people who want objects back, as it will make the C<API> and the code much cleaner )
+( In fact, I may even make 2 specific sub-classes of C<PDD> for people who want objects back, as it will make the C<API> and the
+code much cleaner )
 
 =head1 AUTHOR
 
@@ -557,7 +935,7 @@ Kent Fredric <kentnl@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2013 by Kent Fredric <kentnl@cpan.org>.
+This software is copyright (c) 2014 by Kent Fredric <kentnl@cpan.org>.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
